/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse für einfache Aufgaben, bei denen es ein auditives Feedback gibt
*/
function AudioFeedbackTask() {
    AudioFeedbackTask.cache(true);
}

/**
* HTML-Element in DOM-Baum einfügen
* @return Element   Erzeugter Knoten
*/
AudioFeedbackTask.createHtml = ScoTools.createHtml;

/**
* Schaltflächen zum Ein-/Ausschalten der Audioausgabe und Einblenden der Lizenz-Infos anzeigen
* @return undefined
*/
AudioFeedbackTask.prototype.establish = function() {
    // Schaltfläche für Lizenzinfos
    new ScoTools.InfoPageLink();
    // Schaltflächen für Stummgeschaltet/Laut
    var self = this;
    this._btnMute = new ScoTools.ImageLink('Töne ausschalten', 'share/Audio-volume-high.svg', undefined, function() { self.setMuted(true); });
    this._btnUnmute = new ScoTools.ImageLink('Töne einschalten', 'share/Audio-volume-muted.svg', undefined, function() { self.setMuted(false); });
    ScoTools.getFooterSection().appendChild(this._btnMute);
}

/**
* Stummschaltung aktivieren (mute = true) oder deaktivieren (mute = false)
* @return undefined
*/
AudioFeedbackTask.prototype.setMuted = function(
    mute,       // Soll die Stummschaltung aktiviert werden?
    noSave      // Optional: Soll nicht gespeichert werden?
) {
    if (this.muted) {
        if (mute)
            return;
        this._btnMute.replace(this._btnUnmute);
        this.muted = false;
    } else {
        if (!mute)
            return;
        this._btnUnmute.replace(this._btnMute);
        this.muted = true;
    }
    if (!noSave) {
        this.write('preferenceAudio', this.muted ? -1 : 0);
    }
}

/**
* Versuchen, eine Verbindung zur Lernplattform aufzubauen
* @return undefined
*/
AudioFeedbackTask.prototype.connect = function() {
    if (typeof ScoTools.LMSLink == 'function') {
        var self = this;
        this.lmslink = new ScoTools.LMSLink();
        this.lmslink.onnew = (function() {
            self.lmsValuesInit();
        });
        this.lmslink.start();
        if (this.lmslink.ready) {
            if (this.read('preferenceAudio') == -1)
                this.setMuted(true, true);
        }
    } else
        this.lmslink = { 'ready': false };
    if (!this.lmslink.ready)
        this.showMsg('Verbindungsaufnahme zur Lernplattform fehlgeschlagen - Fortschritte können nicht gespeichert werden', 'msg_warn', true);
    this.showFinalMessage();
}

/**
* Beim ersten Start des Lernpaketes Anfangswerte festlegen
* @return undefined
*/
AudioFeedbackTask.prototype.lmsValuesInit = function() {
    this.saveScore(0, 0, 100, 'not attempted');
}

/**
* Anzeigen, falls Aufgabe endgültig erfolgreich abgeschlossen oder endgültig misslungen ist (sofern das nicht ohnehin schon angezeigt wird)
* @return undefined
*/
AudioFeedbackTask.prototype.showFinalMessage = function() {
    if (this._finalMessage)
        return;
    if (this.getDone()) {
        if (this.lmslink.lessonStatus == 'failed')
            this.showMsg('Aufgabe wurde engültig nicht geschafft', false, true);
        else
            this.showMsg('Aufgabe erfolgreich abgeschlossen', true, true);
        this._finalMessage = true;
    }
}

// Genutzte Instanz von LMSLink
AudioFeedbackTask.prototype.lmslink = { 'ready': false };

// Wurde das auditive Feedback stumm geschaltet?
AudioFeedbackTask.prototype.muted = false;

// Abzuspielendes Audio, wenn richtige Lösung zusammengestellt wurde
AudioFeedbackTask.audioSuccess = null;

// Abzuspielendes Audio, wenn eine Lösung falsch ist
AudioFeedbackTask.audioFail = null;

// Zu cachende bzw. später gecachte Grafiken
AudioFeedbackTask.caching = ['not-ok.svg', 'ok.svg'];

/**
* Lädt Audio- und Bilddateien
* @return undefined
*/
AudioFeedbackTask.cache = function(
    boolLoadAudio       // Auch Audiodateien laden?
) {
    if (!AudioFeedbackTask.audioFail && boolLoadAudio)
        AudioFeedbackTask.audioFail = new Audio('share/fail.mp3');
    if (!AudioFeedbackTask.audioSuccess && boolLoadAudio)
        AudioFeedbackTask.audioSuccess = new Audio('share/success.mp3');
    AudioFeedbackTask.caching = AudioFeedbackTask.caching.map(function(filename) {
        if ((typeof filename == 'string') || (filename instanceof String)) {
            var img = new Image();
            img.src = 'share/' + filename;
            return { "filename": filename, "img": img };
        }
        return filename;
    });
}

// MessagePlace-Instanz für generelle Meldungen
AudioFeedbackTask.prototype.msgPlaceGeneral = null

// MessagePlace-Instanz für nicht-generelle Meldungen
AudioFeedbackTask.prototype.msgPlace = null;

/**
* Eine Meldung anzeigen
* @return undefined
*/
AudioFeedbackTask.prototype.showMsg = function(
    text,           // Textinhalt der Meldung
    type,           // Optional: Art der Meldung (false, true oder Wert aus ScoTools.MSGTYPES)
    boolGeneral     // Optional: Ist die Meldung generell und permanent?
) {
    var place = this.msgPlace;
    if (boolGeneral && this.msgPlaceGeneral)
        place = this.msgPlaceGeneral;
    if (place)
        place.add(text, type);
}

/**
* Zuletzt angezeigte Meldung löschen
* @return undefined
*/
AudioFeedbackTask.prototype.clearMsg = function() {
    this.msgPlace.clear();
}

/**
* Prüft, ob eine Verbindung zur Lernplattform besteht
* @return Boolean   Besteht die Verbindung?
*/
AudioFeedbackTask.prototype.isConnected = function() {
    return this.lmslink.ready;
}

/**
* Ermittelt, ob die Aufgabe bereits erfolgreich gelöst wurde
* @return Boolean       Aufgabe bereits gelöst?
*/
AudioFeedbackTask.prototype.getDone = function() {
    return (this.lmslink.ready && !this.lmslink.isIncomplete());
}

/**
* Prüft, ob Veränderungen am Lernpaket gespeichert werden sollen
* @return Boolean       Kann und soll gespeichert werden?
*/
AudioFeedbackTask.prototype.canSave = function() {
    return (!this.lmslink.ready || this.lmslink.isNormalIncomplete());
}

/**
* Bearbeitungszeit speichern
* @return undefined
*/
AudioFeedbackTask.prototype.reportTime = function() {
    this.lmslink.ready && this.lmslink.reportTime();
}

/**
* Punktestand und Status speichern
* @return undefined
*/
AudioFeedbackTask.prototype.saveScore = function(
    scoreRaw,       // Erreichter Punktestand
    scoreMin,       // Minimal erreichbarer Punktestand
    scoreMax,       // Maximal erreichbarer Punktestand
    status          // Optional: lessonStatus
) {
    if (this.canSave())
        this.lmslink.saveScore(scoreRaw, scoreMin, scoreMax, status);
}

/**
* Einen Wert vom LMS lesen
* @return undefined
*/
AudioFeedbackTask.prototype.read = function(
    key     // Attributname von LMSLink-Instanz, die gelesen werden soll
) {
    return (this.lmslink.ready ? this.lmslink[key] : undefined);
}

/**
* Einen Wert vom LMS speichern lassen
* @return undefined
*/
AudioFeedbackTask.prototype.write = function(
    key,        // Attributname von LMSLink-Instanz, unter dem der Wert gespeichert werden soll
    value       // Zu speichernder Wert
) {
    if (this.lmslink.ready)
        this.lmslink[key] = value;
}

/**
* Zugleich mehrere Werte ändern, die Zeit speichern und dann erst Commit an das LMS senden
* @return undefined
*/
AudioFeedbackTask.prototype.save = function(
    data        // Objekt mit zu speichernden Daten
) {
    if (this.lmslink.ready)
        this.lmslink.save(data);
}

/**
* Eine auditive Rückmeldung abspielen (sofern diese verfügbar und erwünscht ist)
* @return undefined
*/
AudioFeedbackTask.prototype.playAudio = function(
    boolSuccess     // Wurde ein Erfolg erzielt?
) {
    if (this.muted)
        return;
    var audio = boolSuccess ? AudioFeedbackTask.audioSuccess : AudioFeedbackTask.audioFail;
    if (audio && audio.play)
        audio.play();
}
